using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using LogicIO.RTCU.DllImport;
using System.IO;
using System.Reflection;

namespace VSMSGW_Csharp_example
{
    class KernelCall
    {
        [System.Runtime.InteropServices.DllImport("Kernel32.dll")]
        public static extern IntPtr LoadLibrary(string path);
    }

    public partial class MainForm : Form
    {
        #region Form control
        /// <summary>
        /// Constructor.
        /// </summary>
        public MainForm()
        {
            try
            {
                // Solve 32/64bit dll problem.
                String src = AppDomain.CurrentDomain.SetupInformation.ApplicationBase + @"LogicIO RTCU library\";
                String dst = src;
                src += (Environment.Is64BitProcess) ? @"64bit\" : @"32bit\";
                foreach (string file in Directory.GetFiles(src))
                {
                    File.Copy(file, Path.Combine(dst, Path.GetFileName(file)), true);
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show("Failed to copy library files\n" + ex.Message);
            }
            InitializeComponent();

             this.Text +=  (Environment.Is64BitProcess) ? " (64bit)" : " (32bit)";
        }

        private String incoming_data = "incoming_data.rtf";
        /// <summary>
        /// Load setting which can't be automaticly bound.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mainForm_Load(object sender, EventArgs e)
        {
            if (File.Exists(incoming_data)) incomingRichTextBox.LoadFile(incoming_data);
            this.Size = Properties.Settings.Default.Size;
            AddToIncomingAndStatus(this.Text + " started.\n", Color.Black);
        }

        /// <summary>
        /// Save data after form has closed.
        /// </summary>
        /// <param name="sender">not used</param>
        /// <param name="e">not used</param>
        private void mainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            AddToIncomingAndStatus(this.Text + " closed.\n", Color.Black);
            incomingRichTextBox.SaveFile(incoming_data);
            Properties.Settings.Default.Size = this.Size;
            Properties.Settings.Default.Save();
        }
        #endregion

        #region General button control functions
        /// <summary>
        /// Clear incoming and status text box
        /// </summary>
        /// <param name="sender">Not used</param>
        /// <param name="e">Not used</param>
        private void clearIncomingButton_Click(object sender, EventArgs e)
        {
            incomingRichTextBox.Clear();
        }

        /// <summary>
        /// Close form button function
        /// </summary>
        /// <param name="sender">Not used</param>
        /// <param name="e">Not used</param>
        private void closeButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Remove user change and return to default configuration.
        /// </summary>
        /// <param name="sender">Not used</param>
        /// <param name="e">Not used</param>
        private void resetSetupButton_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.Reset();
            this.Size = Properties.Settings.Default.Size;
        }
        #endregion

        #region Add data to 'incoming and status'
        /// <summary>
        /// This delegate enables asynchronous calls for adding text.
        /// </summary>
        /// <param name="text">text to add</param>
        /// <param name="color">color of test when added</param>
        delegate void AddColoredTextCallback(String text, Color color);
        public void AddToIncomingAndStatus(String text, Color color)
        {
            if (incomingRichTextBox.InvokeRequired)
            {
                AddColoredTextCallback d = new AddColoredTextCallback(AddToIncomingAndStatus);
                this.incomingRichTextBox.Invoke(d, new object[] { text, color });
            }
            else
            {
                int l = this.incomingRichTextBox.TextLength;
                this.incomingRichTextBox.AppendText(text);
                this.incomingRichTextBox.SelectionStart = l;
                this.incomingRichTextBox.SelectionLength = text.Length;
                this.incomingRichTextBox.SelectionColor = color;
                this.incomingRichTextBox.ScrollToCaret();
            }
        }
        #endregion

        #region Handle incoming RTCU Gateway messages
        /// <summary>
        /// Incoming SMS callback, used to avoid garbage collection
        /// </summary>
        public VSMSGW.cbfuncText ReceiveSMS;
        UInt16 receiveSMS(Int32 HisNodeID, String Message, IntPtr arg)
        {
            AddToIncomingAndStatus("SMS received from " + HisNodeID + " size " + Message.Length + ": " + Message + "\n", Color.Blue);
            return 0;
        }

        /// <summary>
        /// Incoming PDU callback, used to avoid garbage collection
        /// </summary>
        public VSMSGW.cbfuncPDU ReceivePDU;
        UInt16 receivePDU(Int32 HisNodeID, Byte[] data, Int32 length, IntPtr arg)
        {
            AddToIncomingAndStatus("PDU received from " + HisNodeID + " size " + length + "\n", Color.Blue);
            return 0;
        }

        /// <summary>
        /// Incoming DATA callback, used to avoid garbage collection
        /// </summary>
        public VSMSGW.cbfuncPackage ReceivePackage;
        UInt16 receivePackage(Int32 HisNodeID, Byte[] data, Int32 length, IntPtr arg)
        {
            AddToIncomingAndStatus("Data packet received from " + HisNodeID + " size " + length + "\n", Color.Blue);
            return 0;
        }
        #endregion

        #region Button controls using VSMSGW
        private void connectButton_Click(object sender, EventArgs e)
        {
            UInt32 MyNodeID = Convert.ToUInt32(Properties.Settings.Default.GWNodeID);
            String GWIP = Properties.Settings.Default.GWIP;
            Int32 GWPort = Convert.ToInt32(Properties.Settings.Default.GWPort);
            String GWKey = Properties.Settings.Default.GWKey;
            Byte[] CryptKey = new byte[16]; // no encryption used

            try
            {
                // try to initialize
                GWRC gwrc = VSMSGW.InitAdv(MyNodeID, GWIP, GWPort, GWKey, CryptKey);
                if ((gwrc != GWRC.OK))
                {
                    AddToIncomingAndStatus("Failed to initialize gateway library (" + gwrc.ToString() + ")\n", Color.Red);
                }
                else
                {

                    AddToIncomingAndStatus("Connecting to '" + Properties.Settings.Default.GWIP + ":" + Properties.Settings.Default.GWPort.ToString() + "'\n", Color.Black);

                    ReceivePDU = new VSMSGW.cbfuncPDU(receivePDU);
                    ReceiveSMS = new VSMSGW.cbfuncText(receiveSMS);
                    ReceivePackage = new VSMSGW.cbfuncPackage(receivePackage);

                    VSMSGW.SetSMSPDUCB(ReceivePDU, IntPtr.Zero);
                    VSMSGW.SetSMSTextCB(ReceiveSMS, IntPtr.Zero);
                    VSMSGW.SetDataPackageCB(ReceivePackage, IntPtr.Zero);
                }
            }
            catch (Exception ex)
            {
                AddToIncomingAndStatus("Fatal error : " + ex.Message + "\n", Color.Red);
            }
        }

        private void checkConnectionButton_Click(object sender, EventArgs e)
        {
            try
            {
                if (VSMSGW.IsConnected())
                {
                    AddToIncomingAndStatus("Status : connected to gateway\n", Color.Green);
                }
                else
                {
                    AddToIncomingAndStatus("Status : NOT connected to gateway\n", Color.Black);
                }

            }
            catch (Exception ex)
            {
                AddToIncomingAndStatus("Fatal error : " + ex.Message + "\n", Color.Red);
            }
        }

        private void getIDButton_Click(object sender, EventArgs e)
        {
            try
            {
                UInt32 MyID = 0;
                GWRC gwrc = VSMSGW.GetMyNodeID(ref MyID);
                if (gwrc != GWRC.OK)
                {
                    AddToIncomingAndStatus("Failed to retrieve id (" + gwrc.ToString() + ")\n", Color.Red);
                }
                else
                {
                    AddToIncomingAndStatus("Connected with id '" + MyID + "'\n", Color.Black);
                    if (!Convert.ToDecimal(MyID).Equals(Properties.Settings.Default.GWNodeID))
                    {
                        AddToIncomingAndStatus("Using new id as default\n", Color.Black);
                        Properties.Settings.Default.GWNodeID = Convert.ToDecimal(MyID);
                    }
                }
            }
            catch (Exception ex)
            {
                AddToIncomingAndStatus("Fatal error : " + ex.Message + "\n", Color.Red);
            }
        }

        private void sendAsSMSButton_Click(object sender, EventArgs e)
        {
            try
            {
                UInt32 hisId = Convert.ToUInt32(Properties.Settings.Default.HisID);
                String text = OUTText.Text;
                Int32 rc = 0;

                GWRC gwrc = VSMSGW.SendSMS(hisId, text, ref rc);
                if (gwrc != GWRC.OK)
                {
                    AddToIncomingAndStatus("Failed to send as SMS (" + gwrc.ToString() + ")\n", Color.Red);
                }
                else
                {
                    AddToIncomingAndStatus("Result of sending '" + text + "' : " + rc.ToString() + "\n", Color.Black);
                }
            }
            catch (Exception ex)
            {
                AddToIncomingAndStatus("Fatal error : " + ex.Message + "\n", Color.Red);
            }
        }

        private void sendAsPDUButton_Click(object sender, EventArgs e)
        {
            try
            {
                UInt32 hisId = Convert.ToUInt32(Properties.Settings.Default.HisID);
                String text = OUTText.Text;
                Byte[] data = Encoding.ASCII.GetBytes(text);
                Int32 rc = 0;

                GWRC gwrc = VSMSGW.SendPDU(hisId, data, data.Length, ref rc);
                if (gwrc != GWRC.OK)
                {
                    AddToIncomingAndStatus("Failed to send as PDU (" + gwrc.ToString() + ")\n", Color.Red);
                }
                else
                {
                    AddToIncomingAndStatus("Result of sending '" + text + "' as PDU: " + rc.ToString() + "\n", Color.Black);
                }
            }
            catch (Exception ex)
            {
                AddToIncomingAndStatus("Fatal error : " + ex.Message + "\n", Color.Red);
            }
        }

        private void sendAsPkgButton_Click(object sender, EventArgs e)
        {
            try
            {
                UInt32 hisId = Convert.ToUInt32(Properties.Settings.Default.HisID);
                String text = OUTText.Text;
                Byte[] data = Encoding.ASCII.GetBytes(text);
                Int32 rc = 0;

                GWRC gwrc = VSMSGW.SendDataPackage(hisId, data, data.Length, ref rc);
                if (gwrc != GWRC.OK)
                {
                    AddToIncomingAndStatus("Failed to send as data package (" + gwrc.ToString() + ")\n", Color.Red);
                }
                else
                {
                    AddToIncomingAndStatus("Result of sending '" + text + "' as data package: " + rc.ToString() + "\n", Color.Black);
                }
            }
            catch (Exception ex)
            {
                AddToIncomingAndStatus("Fatal error : " + ex.Message + "\n", Color.Red);
            }
        }
        #endregion
    }
}